#include <stdio.h>
#include <stdlib.h>  // Include for malloc and free
#include <mpi.h>

// Function to calculate the partial area under the curve X^2 for a given range
double calculate_partial_area(double start, double end, int num_steps, int rank, int size) {
    double width = (end - start) / num_steps;
    double partial_area = 0.0;

    // Loop over the subintervals assigned to the current process
    for (int i = rank; i < num_steps; i += size) {
        double x = start + i * width;
        double height = x * x;
        partial_area += height * width;
    }

    return partial_area;
}

int main(int argc, char *argv[]) {
    MPI_Init(&argc, &argv);

    int rank, size;
    MPI_Comm_rank(MPI_COMM_WORLD, &rank);
    MPI_Comm_size(MPI_COMM_WORLD, &size);

    int num_steps = 1000000;  // Adjust based on the required precision

    // Measure execution time
    double start_time = MPI_Wtime();

    // Calculate the partial area for the current process
    double partial_area = calculate_partial_area(0.0, 1.0, num_steps, rank, size);

    double *all_partial_areas = NULL;
    if (rank == 0) {
        // Allocate memory for storing partial areas at the root process
        all_partial_areas = (double *)malloc(size * sizeof(double));
    }

    // Gather the partial areas from all processes to the root process
    MPI_Gather(&partial_area, 1, MPI_DOUBLE, all_partial_areas, 1, MPI_DOUBLE, 0, MPI_COMM_WORLD);

    // Stop measuring time
    double end_time = MPI_Wtime();
    double elapsed_time = end_time - start_time;

    if (rank == 0) {
        // Sum up the partial areas to get the total area
        double total_area = 0.0;
        for (int i = 0; i < size; i++) {
            total_area += all_partial_areas[i];
        }

        // Print the results
        printf("Total Area (MPI): %f\n", total_area);
        printf("Time: %f seconds\n", elapsed_time);

        // Free the allocated memory at the root process
        free(all_partial_areas);
    }

    MPI_Finalize();

    return 0;
}
