#include <stdio.h>
#include <omp.h>

// Function to calculate the partial area under the curve X^2 for a given range
double calculate_partial_area(double start, double end, int num_steps) {
    double width = (end - start) / num_steps;
    double partial_area = 0.0;

    // Parallel region with private_partial_area for each thread
    #pragma omp parallel
    {
        double private_partial_area = 0.0;

        // Parallel loop to calculate partial areas for each thread
        #pragma omp for
        for (int i = 0; i < num_steps; i++) {
            double x = start + i * width;
            double height = x * x;
            private_partial_area += height * width;
        }

        // Combine the partial areas from each thread using a critical section
        #pragma omp critical
        partial_area += private_partial_area;
    }

    return partial_area;
}

int main() {
    int num_steps = 1000000;  // Adjust based on the required precision

    // Measure execution time
    double start_time = omp_get_wtime();

    // Calculate the total area using OpenMP parallelization
    double total_area = calculate_partial_area(0.0, 1.0, num_steps);

    // Stop measuring time
    double end_time = omp_get_wtime();
    double elapsed_time = end_time - start_time;

    // Print the results
    printf("Total Area (OpenMP): %f\n", total_area);
    printf("Time: %f seconds\n", elapsed_time);

    return 0;
}
